<?php
/*
 * $Horde: whups/lib/Forms.php,v 1.19 2002/03/02 19:16:40 chuck Exp $
 *
 * Copyright 2001 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 */

/**
 * Type Class
 *
 * @author Robert E. Coyle <robertecoyle@hotmail.com>
 * @version $Revision: 1.19 $
 * @package whups
 */
class Type {
    function isValid(&$var, &$vars, $value, &$message) {
        $message = "<b>Error:</b> Type::isValid called - should be overridden<br />";
        return false;
    }

    function getTypeName() {
        return strtolower(substr(get_class($this), 0, -4));
    }

    function getValues() {
        return null;
    }
}

class IntType extends Type {
    function isValid(&$var, &$vars, $value, &$message)
    {
        if (empty($value) || preg_match('/^[0-9]+$/', $value)) {
            return true;
        }

        $message = _("may only contain numbers");
        return false;
    }
}

class IntListType extends Type {
    function isValid(&$var, &$vars, $value, &$message) {
        if (empty($value) || preg_match('/^[0-9 ,]+$/', $value))
            return true;

        $message = _("must be a comma or space separated list of numbers");
        return false;
    }
}

class TextType extends Type {
    function isValid(&$var, &$vars, $value, &$message) {
        $valid = true;

        if ($var->isRequired()) {
            $valid = strlen(trim($value)) > 0;

            if (!$valid) {
                $message = sprintf(_("%s is required"), $var->getHumanName());
            }
        }

        return $valid;
    }
}

class LongTextType extends Type {
    function isValid(&$var, &$vars, $value, &$message) {
        $valid = true;

        if ($var->isRequired()) {
            $valid = strlen(trim($value)) > 0;

            if (!$valid) {
                $message = sprintf(_("%s is required"), $var->getHumanName());
            }
        }

        return $valid;
    }
}

class BooleanType extends Type {
    function isValid(&$var, &$vars, $value, &$message) {
        return true;
    }
}

class LinkType extends Type {

    function LinkType($values)
    {
        $this->values = $values;
    }

    function isValid(&$var, &$vars, $value, &$message)
    {
        return true;
    }
}

class EmailType extends Type {
    function isValid(&$var, &$vars, $value, &$message) {
        // FIXME: Should also check for a valid email address
        $valid = true;

        if ($var->isRequired()) {
            $valid = strlen(trim($value)) > 0;

            if (!$valid) {
                $message = sprintf(_("%s is required"), $var->getHumanName());
            }
        }

        return $valid;
    }
}

class EnumType extends Type {

    var $values;
    
    function EnumType($values)
    {
        $this->values = $values;
    }

    function isValid(&$var, &$vars, $value, &$message)
    {
        if (count($this->values) == 0 || isset($this->values[$value])) {
            return true;
        }
        
        $message = _("hey, stop screwing with my post vars");
        return false;
    }

    function getValues()
    {
        return $this->values;
    }

}

class SetType extends Type {

    var $values;
    
    function SetType(&$values)
    {
        $this->values = $values;
    }

    function isValid(&$var, &$vars, $value, &$message)
    {
        if (count($this->values) == 0 || count($value) == 0) {
            return true;
        }
        foreach ($value as $item) {
            if (!isset($this->values[$item])) {
                $error = true;
                break;
            }
        }
        if (!isset($error)) {
            return true;
        }
        
        $message = _("hey, stop screwing with my post vars");
        return false;
    }

    function getValues()
    {
        return $this->values;
    }

}

class InvalidType extends Type {

    var $message;
    
    function InvalidType($message)
    {
        $this->message = $message;
    }

    function isValid(&$var, &$vars, $value, &$message)
    {
        return false;
    }

}

class Variable {

    function Variable($humanName, $varName, &$type, $required, $readonly = false)
    {
        $this->humanName = $humanName;
        $this->varName   = $varName;
        $this->type      = $type;
        $this->required  = $required;
        $this->readonly  = $readonly;
        $this->defValue  = null;
    }

    function setDefault($value) { $this->defValue = $value; }

    function  getHumanName() {return $this->humanName;}
    function  getVarName() {return $this->varName;}
    function &getType() {return $this->type;}
    function  getDefault() { return $this->defValue; }
    function  isRequired() {return $this->required;}
    function  isReadonly() {return $this->readonly;}
    function  getTypeName() {return $this->type->getTypeName();}
    function  getValues() {return $this->type->getValues();}

    function validate(&$vars, &$message)
    {
        $value = $this->getValue($vars);
        return $this->type->isValid($this, $vars, $value, $message);
    }

    function getValue(&$vars)
    {
        $value = $vars->getVarWasset($this->varName, $wasset);

        if (!$wasset) {
            $value = $this->getDefault();
        }

        return $value;
    }

}

// custom renderer ABC
class CustomRenderer {
    function render(&$form, &$vars, $isActive) {
        debug("Warning: CustomRenderer::render called, should be overridden");
    }
}

class Form {

    function Form(&$vars, $title = '', $name = null)
    {
        if ($name === null) {
            $name = get_class($this);
        }

        $this->name = $name;
        $this->title = $title;
        $this->vars = &$vars;
        $this->errors = array();
        $this->variables = array();
        $this->hiddenVariables = array();
    }

    function getTitle()
    {
        return $this->title;
    }

    function getName()
    {
        return $this->name;
    }

    // protected
    function addVariable(&$var) {
        if ($var->getTypeName() == 'enum' &&
            count($var->getValues()) == 1) {
            $vals = array_keys($var->getValues());
            $this->vars->addVar($var->varName, $vals[0]);
        }
        array_push($this->variables, $var);
    }

    // protected
    function addHidden(&$var) {
        array_push($this->hiddenVariables, $var);
    }

    function &getVariables() {
        return $this->variables;
    }

    function open(&$renderer, &$vars, $action, $method = 'get') {
        $renderer->open($action, $method, $this->name);

        if (!empty($this->name)) {
            preserveVarByPost('formname', $this->name);
        }

        $vars->preserveVarByPost('action');

        foreach ($this->hiddenVariables as $key => $var) {
            $vars->preserveVarByPost($var->getVarName());
        }

        $vars->pushPrefix('debug');
        $vars->preserveByPost();
        $vars->popPrefix();
    }

    function getCustomRenderer() {
        return null;
    }

    function preserve($vars) {
        foreach ($this->variables as $key => $var) {
            $vars->preserveVarByPost($var->getVarName());
        }
        foreach ($this->hiddenVariables as $key => $var) {
            $vars->preserveVarByPost($var->getVarName());
        }
    }

    function close(&$renderer) {
        $renderer->close();
    }

    function unsetVars(&$vars) {
        foreach ($this->variables as $key => $var) {
            $varname = $var->getVarName();
            $vars->unsetVar($varname);
        }
    }

    function validate(&$vars) {
        $message = '';

        foreach ($this->variables as $key => $var) {
            if (!$var->validate($vars, $message)) {
                $this->errors[$var->getVarName()] = $message;
            }
        }
        foreach ($this->hiddenVariables as $key => $var) {
            if (!$var->validate($vars, $message)) {
                $this->errors[$var->getVarName()] = $message;
            }
        }
    }

    function clearValidation()
    {
        $this->errors = array();
    }

    function getError(&$var) {
        $name = $var->getVarName();
        return isset($this->errors[$name]) ? $this->errors[$name] : null;
    }

    function isValid() {
        return (count($this->errors) == 0);
    }

    function execute(&$vars) {
        debug("Warning: Form::execute called, should be overridden");
    }

    // delegates to _getInfo so that subclasses can override this but still call the
    // superclass method to create the default info array
    function getInfo(&$vars, &$info) {
        $this->_getInfo($vars, $info);
    }

    function _getInfo(&$vars, &$info) {
        $this->_getInfoFromVariables($this->variables, $vars, $info);
        $this->_getInfoFromVariables($this->hiddenVariables, $vars, $info);
    }

    function _getInfoFromVariables(&$variables, &$vars, &$info) {
        foreach ($variables as $key => $var) {
            $varname = $var->getVarName();
            $value = $vars->getVarWasset($varname, $wasset);

            if ($wasset) {
                $info[$varname] = $value;
            } else if (isset($var->defValue)) {
                $info[$varname] = $var->defValue;
            }
        }
    }

}

function useTypes($types) {
    foreach ($types as $t) {
        eval("function &get$t() {static \$t = ''; if (empty(\$t)) \$t = new $t(); return \$t;}");
    }
}

useTypes(array(
    'IntType',
    'IntListType',
    'TextType',
    'LongTextType',
    'BooleanType',
    'EmailType'));

?>
